@@ -184,9 +184,9 @@ short pagelinks(int page)
 short *pagetotals(int page, short mode)
 {
    int i;
-   short *counts, *icount, *result;
+   short *counts, *icount; /* , *result; (jdk) */
 
-   if (xobjs.pagelist[page]->pageinst == NULL) return;
+   if (xobjs.pagelist[page]->pageinst == NULL) return NULL;
 
    counts = (short *)malloc(xobjs.pages * sizeof(short));
    icount = (short *)malloc(xobjs.pages * sizeof(short));
@@ -297,30 +297,18 @@ int is_library(objectptr thisobj)
    return -1;
 }
 
-/*------------------------------------------------------*/
-/* Check if a library (given by name) exists.  If it	*/
-/* does, return the library number.  If not, return -1	*/
-/*------------------------------------------------------*/
-
-int check_library(char *libname)
-{
-   int i;
-   
-   for (i = 0; i < xobjs.numlibs; i++)
-      if (!strcmp(xobjs.libtop[i + LIBRARY]->thisobject->name, libname))
-	 return i;
-
-   return -1;
-}
-
 /*------------------------------------------------------*/
 /* Check for library name (string).  Because XCircuit   */
 /* generates the text "Library: <filename>" for		*/
 /* library names, we also check against <filename>	*/
-/* only in these names.					*/
+/* only in these names (this library name syntax is	*/
+/* deprecated, but the check is retained for backwards	*/
+/* compatibility).					*/
+/*							*/
+/* If no library matches the given name, return -1.	*/
 /*------------------------------------------------------*/
 
-int name_to_library(char *libname)
+int NameToLibrary(char *libname)
 {
    char *slib;
    int i;
@@ -891,6 +879,7 @@ void composelib(short mode)
 
       if (fval < fontcount) {
 	 stringpart *strptr;
+	 /* char *locobjname; (jdk) */
 
 	 NEW_LABEL(drawname, libpage);
          libpage->parts++;
@@ -901,6 +890,7 @@ void composelib(short mode)
 	 (*drawname)->string->data.font = fval;
 	 strptr = makesegment(&((*drawname)->string), NULL);
 	 strptr->type = TEXT_STRING;
+
          strptr->data.string = strdup(libobj->name);
          (*drawname)->justify = TOP | NOTBOTTOM | NOTLEFT;
 
@@ -995,11 +985,11 @@ void catvirtualcopy()
       libobj = SELTOOBJINST(newselect);
       libinst = addtoinstlist(i, libobj->thisobject, TRUE);
       instcopy(libinst, libobj);
+      tech_mark_changed(GetObjectTechnology(libobj->thisobject));
    }
 
    clearselects();
    composelib(LIBRARY + i);
-   lib_mark_changed(i);
    drawarea(NULL, NULL, NULL);
 }
 
@@ -1046,7 +1036,7 @@ void catdelete()
 {
    short *newselect, *libpobjs;
    int i;
-   genericptr *testobj, *tobj;
+   /* genericptr *testobj, *tobj; (jdk) */
    objinstptr libobj;
    liblistptr ilist, llist;
    objectptr *libpage, *compobj, *tlib, *slib;
@@ -1125,6 +1115,7 @@ void catdelete()
 	 }
 
 	 /* Finally, delete the object (permanent---no undoing this!) */
+         tech_mark_changed(GetObjectTechnology(libobj->thisobject));
 	 reset(libobj->thisobject, DESTROY);
       }
    }
@@ -1133,7 +1124,6 @@ void catdelete()
 
    if ((i = is_library(topobject)) >= 0) {
       composelib(LIBRARY + i);
-      lib_mark_changed(i);
    }
 
    drawarea(NULL, NULL, NULL);
@@ -1183,7 +1173,7 @@ void linkedlistswap(liblistptr *spec, int o1, int o2)
 
 void linkedlistinsertafter(liblistptr *spec, int o1, int o2)
 {
-   liblistptr s1, s1m, s2, s2m, stmp;
+  liblistptr s1, s1m, s2; /* , s2m, stmp; (jdk) */
    int j;
 
    if ((o1 == o2) || (o1 == (o2 + 1))) return;
@@ -1217,32 +1207,37 @@ void linkedlistinsertafter(liblistptr *spec, int o1, int o2)
 /*------------------------------------------------------*/
 /* Set the "changed" flag in a library if any object	*/
 /* in that library has changed.				*/
+/*							*/
+/* If "technology" is NULL, check all objects,		*/
+/* otherwise only check objects with a matching		*/
+/* technology.						*/
 /*------------------------------------------------------*/
 
-void lib_set_changes(libnum)
+void tech_set_changes(TechPtr refns)
 {
-   int j;
+   TechPtr ns;
+   int i, j;
    objectptr thisobj;
 
-   if ((xobjs.userlibs[libnum].flags & LIBRARY_CHANGED) != 0)
-      return;
-
-   for (j = 0; j < xobjs.userlibs[libnum].number; j++) {
-      thisobj = *(xobjs.userlibs[libnum].library + j);
-      if (getchanges(thisobj) > 0) {
-	 xobjs.userlibs[libnum].flags |= LIBRARY_CHANGED;
-	 break;
+   for (i = 0; i < xobjs.numlibs; i++) {
+      for (j = 0; j < xobjs.userlibs[i].number; j++) {
+         thisobj = *(xobjs.userlibs[i].library + j);
+         if (getchanges(thisobj) > 0) {
+            ns = GetObjectTechnology(thisobj);
+	    if ((refns == NULL) || (refns == ns)) 
+	       ns->flags |= LIBRARY_CHANGED;
+	 }
       }
    }
 }
 
 /*------------------------------------------------------*/
-/* Mark a library as having been modified.		*/
+/* Mark a technology as having been modified.		*/
 /*------------------------------------------------------*/
 
-void lib_mark_changed(int libnum)
+void tech_mark_changed(TechPtr ns)
 {
-   xobjs.userlibs[libnum].flags |= LIBRARY_CHANGED;
+   if (ns != NULL) ns->flags |= LIBRARY_CHANGED;
 }
 
 /*------------------------------------------------------*/
@@ -1251,7 +1246,7 @@ void lib_mark_changed(int libnum)
 
 void catmove(int x, int y)
 {
-   int i, j, k, s1, s2, ocentx, ocenty, rangex, rangey, l;
+  int i, j, k, s1, s2, ocentx, ocenty, rangey, l; /* rangex, (jdk) */
    liblistptr spec;
    objinstptr exchobj, lobj;
 
@@ -1340,19 +1335,18 @@ void catmove(int x, int y)
 
    unselect_all();
    composelib(LIBRARY + i);
-   lib_mark_changed(i);
    if (j != i) {
       composelib(LIBRARY + j);
       centerview(xobjs.libtop[LIBRARY + j]);
-      lib_mark_changed(j);
    }
 
    drawarea(NULL, NULL, NULL);
 }
 
-/*--------------------------------------------------------*/
-/* Make a duplicate of an object, put in the User Library */
-/*--------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Make a duplicate of an object, put in the User	*/
+/* Library or the current library (if we're in one).	*/
+/*------------------------------------------------------*/
 
 void copycat()
 {
@@ -1360,7 +1354,10 @@ void copycat()
    objectptr *newobj, *curlib, oldobj;
    objinstptr libobj;
    oparamptr ops, newops;
-   int i, libnum = USERLIB - LIBRARY ;
+   int i, libnum;
+
+   libnum = is_library(topobject);
+   if (libnum < 0) libnum = USERLIB - LIBRARY;  /* default */
 
    for (newselect = areawin->selectlist; newselect < areawin->selectlist
 	  + areawin->selects; newselect++) {
@@ -1441,41 +1438,11 @@ void copycat()
 	 switch(ELEMENTTYPE(*(oldobj->plist + i))) {
 	    case(PATH): {
 	       register pathptr *npath, cpath = TOPATH(oldobj->plist + i);
-	       register genericptr *gpart;
+
 	       NEW_PATH(npath, (*newobj));
-	       (*npath)->style = cpath->style;
-	       (*npath)->width = cpath->width;
-	       (*npath)->parts = 0;
-	       (*npath)->plist = (genericptr *)malloc(cpath->parts *
-			sizeof(genericptr));
-               (*npath)->passed = NULL;
-	       for (gpart = cpath->plist; gpart < cpath->plist + cpath->parts;
-			gpart++) {
-		  switch(ELEMENTTYPE(*gpart)){
-		     case ARC: {
-			arcptr copyarc = TOARC(gpart);
-			arcptr *newarc;
-			NEW_ARC(newarc, (*npath));
-			arccopy(*newarc, copyarc);
-			(*npath)->parts++;
-			} break;
-		     case POLYGON: {
-                        polyptr copypoly = TOPOLY(gpart);
-                        polyptr *newpoly;
-                        NEW_POLY(newpoly, (*npath));
-                        polycopy(*newpoly, copypoly);
-                        (*npath)->parts++;
-                        } break;
-                     case SPLINE: {
-                        splineptr copyspline = TOSPLINE(gpart);
-                        splineptr *newspline;
-                        NEW_SPLINE(newspline, (*npath));
-                        splinecopy(*newspline, copyspline);
-                        (*npath)->parts++;
-                        } break;
-		  }
-	       }	
+	       pathcopy(*npath, cpath);
 	       } break;
+
 	    case(ARC): {
 	       register arcptr *narc, carc = TOARC(oldobj->plist + i);
 
@@ -1484,33 +1451,29 @@ void copycat()
                } break;
       
 	    case(POLYGON): {
-	       register polyptr *npoly, cpoly;
+	       register polyptr *npoly, cpoly = TOPOLY(oldobj->plist + i);
 
-	       cpoly = TOPOLY(oldobj->plist + i);
 	       NEW_POLY(npoly, (*newobj));
 	       polycopy(*npoly, cpoly);
                } break;
 
 	    case(SPLINE): {
-	       register splineptr *nspl, cspl;
+	       register splineptr *nspl, cspl = TOSPLINE(oldobj->plist + i);
 
-	       cspl = TOSPLINE(oldobj->plist + i);
 	       NEW_SPLINE(nspl, (*newobj));
 	       splinecopy(*nspl, cspl);
       	       } break;
 
 	    case(LABEL): {
-	       register labelptr *nlabel, clabel;
+	       register labelptr *nlabel, clabel = TOLABEL(oldobj->plist + i);
 
-	       clabel = TOLABEL(oldobj->plist + i);
 	       NEW_LABEL(nlabel, (*newobj));
 	       labelcopy(*nlabel, clabel);
                } break;
       
 	    case(OBJINST): {
-	       register objinstptr *ninst, cinst;
+	       register objinstptr *ninst, cinst = TOOBJINST(oldobj->plist + i);
 
-	       cinst = TOOBJINST(oldobj->plist + i);
 	       NEW_OBJINST(ninst, (*newobj));
 	       instcopy(*ninst, cinst);
                } break;
@@ -1523,7 +1486,6 @@ void copycat()
    addtoinstlist(USERLIB - LIBRARY, *newobj, FALSE);
 
    composelib(USERLIB);
-   lib_mark_changed(USERLIB - LIBRARY);
    unselect_all();
 
    if (areawin->topinstance == xobjs.libtop[USERLIB])
@@ -1663,7 +1625,11 @@ void catalog_op(int op, int x, int y)
 	          }
 	          else {
                      eventmode = MOVE_MODE;
-	             areawin->editcycle = 1;
+		     /* This is a hack, but avoids having to define a	*/
+		     /* separate event type for a move following a	*/
+		     /* library selection, just so we can handle	*/
+		     /* "cancel" differently.				*/
+	             areawin->editpart = -1;
 	          }
 #ifdef TCL_WRAPPER
 		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
@@ -1761,7 +1727,6 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
    /* to a library from the library directory or vice versa, or	    */
    /* library to library.					    */
 
-
    if (!(((is_library(topobject) >= 0)
 		|| (areawin->topinstance == xobjs.libtop[LIBLIB])
 		|| (areawin->topinstance == xobjs.libtop[PAGELIB]))
@@ -1775,7 +1740,6 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
    topobject->pcorner = areawin->pcorner;
    areawin->topinstance = xobjs.libtop[libmod];
    setpage(TRUE);
-   invalidate_graphics(topobject);
 
    transferselects();
 
