@@ -7,11 +7,11 @@
 
 /* Note variable argument list for register_for_undo() */
 extern void register_for_undo(u_int, u_char, objinstptr, ...);
-extern void undo_action();
-extern void redo_action();
-extern void flush_undo_stack();
-extern void flush_redo_stack();
-extern void truncate_undo_stack();
+extern void undo_action(void);
+extern void redo_action(void);
+extern void flush_undo_stack(void);
+extern void flush_redo_stack(void);
+extern void truncate_undo_stack(void);
 extern void free_undo_record(Undoptr);
 extern void free_redo_record(Undoptr);
 extern stringpart *get_original_string(labelptr);
@@ -28,17 +28,19 @@ extern void redo_call(xcWidget, caddr_t, caddr_t);
 /* from tclxcircuit.c */
 
 #ifdef TCL_WRAPPER
-extern Tk_EventProc *xctk_drag(ClientData, XEvent *);
-extern Tk_EventProc *xctk_draglscroll(ClientData, XEvent *);
-extern Tk_EventProc *xctk_dragfilebox(ClientData, XEvent *);
+extern void xctk_drag(ClientData, XEvent *);
+extern void xctk_draglscroll(ClientData, XEvent *);
+extern void xctk_dragfilebox(ClientData, XEvent *);
+extern void tcl_stdflush(FILE *);
+extern void tcl_printf(FILE *, const char *, ...);
 extern int XcTagCallback(Tcl_Interp *, int, Tcl_Obj *CONST objv[]);
-extern Tcl_Obj *evaluate_raw(oparamptr, int *);
+extern Tcl_Obj *evaluate_raw(objectptr, oparamptr, objinstptr, int *);
 extern char *TCL_to_PS(char *);
 extern XCWindowData *GUI_init(int, Tcl_Obj *CONST objv[]);
 extern void build_app_database(Tk_Window);
 #endif
 
-extern char *evaluate_expr(oparamptr, objinstptr);
+extern char *evaluate_expr(objectptr, oparamptr, objinstptr);
 
 /* from opengl.c:  compatibility functions for OpenGL/X11 */
 
@@ -86,17 +88,18 @@ extern void copyparams(objinstptr, objinstptr);
 extern void textbutton(u_char, int, int);
 extern void charreport(labelptr);
 extern Boolean labeltext(int, char *);
-extern void textreturn();
+extern void textreturn(void);
 extern void rejustify(short);
 extern void finddir(polyptr);
 extern void reversepoints(XPoint *, short);
 extern void reversefpoints(XfPoint *, short);
 extern void freeparts(short *, short);
 extern void removep(short *, short);
-extern void unjoin();
+extern void unjoin(void);
 extern labelptr findlabelcopy(labelptr, stringpart *);
 extern Boolean neartest(XPoint *, XPoint *);
-extern void join();
+extern void join(void);
+extern genericptr getsubpart(pathptr);
 
 /* interactive manipulation of elements */
 extern void splinebutton(int, int);
@@ -106,17 +109,18 @@ extern void trackarc(xcWidget, caddr_t, caddr_t);
 extern void boxbutton(int, int);
 extern void trackbox(xcWidget, caddr_t, caddr_t);
 extern void trackwire(xcWidget, caddr_t, caddr_t);
-extern void startwire(XPoint);
+extern void startwire(XPoint *);
 extern void trackpoly(xcWidget, caddr_t, caddr_t);
+extern void trackpath(xcWidget, caddr_t, caddr_t);
 extern void setendpoint(short *, short, XPoint **, XPoint *);
 extern void wire_op(int, int ,int);
 
 /* from events.c: */
 
 extern Boolean recursefind(objectptr, objectptr);
-extern void transferselects();
-extern void select_invalidate_netlist();
-extern void newmatrix();
+extern void transferselects(void);
+extern void select_invalidate_netlist(void);
+extern void newmatrix(void);
 extern void setpage(Boolean);
 extern int changepage(short);
 extern void newpage(short);
@@ -141,8 +145,10 @@ extern void zoominrefresh(int, int);
 extern void zoomoutrefresh(int, int);
 extern void panrefresh(u_int, int, int, float);
 extern void checkwarp(XPoint *);
-extern int checkcycle(short, short);
-extern void copyvirtual();
+extern void warparccycle(arcptr);
+extern int checkcycle(genericptr, short);
+extern void copyvirtual(void);
+extern void nextpathcycle(pathptr, short);
 extern void nextpolycycle(polyptr, short);
 extern void nextsplinecycle(splineptr, short);
 extern void nextarccycle(arcptr, short);
@@ -153,7 +159,7 @@ extern int eventdispatch(int, int, int);
 extern int functiondispatch(int, short, int, int);
 extern void releasehandler(xcWidget, caddr_t, XKeyEvent *);
 extern void setsnap(short);
-extern void snapelement();
+extern void snapelement(void);
 extern int ipow10(int);
 extern int calcgcf(int, int);
 extern void fraccalc(float, char *);
@@ -167,47 +173,54 @@ extern void xlib_drag(xcWidget, caddr_t, XEvent *);
 extern void elemrotate(genericptr *, short, XPoint *);
 extern void elementrotate(short, XPoint *);
 extern void edit(int, int);
-extern void pathedit(genericptr *, short);
+extern void pathedit(genericptr);
 extern void xc_lower(short *);
 extern void xc_raise(short *);
-extern void exchange();
+extern void exchange(void);
 extern void elhflip(genericptr *, short);
 extern void elvflip(genericptr *, short);
 extern void elementflip(XPoint *);
 extern void elementvflip(XPoint *);
+extern short getkeynum(void);
 #ifdef TCL_WRAPPER
-extern xcTimeOutProc makepress(caddr_t);
+extern void makepress(ClientData);
 #else
-extern xcTimeOutProc makepress(caddr_t, xcIntervalId *);
+extern void makepress(XtPointer, xcIntervalId *);
 #endif
 extern void reviseselect(short *, int, short *);
 extern void deletebutton(int, int);
+extern void delete_one_element(objinstptr, genericptr);
 extern short *xc_undelete(objinstptr, objectptr, short, short *);
 extern objectptr delete_element(objinstptr, short *, int, short);
 extern void printname(objectptr);
 extern Boolean checkname(objectptr);
-extern char *checkpostscriptname(char *, int *, objectptr);
-extern objectptr finddot();
+extern char *checkvalidname(char *, objectptr);
+extern objectptr finddot(void);
 extern void movepoints(genericptr *, short, short);
 #ifndef TCL_WRAPPER
 extern void xlib_makeobject(xcWidget, caddr_t);
 #endif
-extern void domakeobject(int);
+extern objinstptr domakeobject(int, char *, Boolean);
 extern void selectsave(xcWidget, caddr_t, caddr_t);
 extern void arceditpush(arcptr);
 extern void splineeditpush(splineptr);
 extern void polyeditpush(polyptr);
+extern void patheditpush(pathptr);
 extern pointlist copypoints(pointlist, int);
 extern void labelcopy(labelptr, labelptr);
 extern void arccopy(arcptr, arcptr);
 extern void polycopy(polyptr, polyptr);
 extern void splinecopy(splineptr, splineptr);
+extern void pathcopy(pathptr, pathptr);
 extern void instcopy(objinstptr, objinstptr);
+extern void delete_tagged(objectptr);
+extern void createcopies(void);
+extern void copydrag(void);
 extern void copy_op(int, int, int);
 extern Boolean checkmultiple(XButtonEvent *);
 extern void continue_op(int, int, int);
 extern void finish_op(int, int, int);
-extern void path_op(genericptr *, int, int, int);
+extern void path_op(genericptr, int, int, int);
 extern void resizearea(xcWidget, caddr_t, caddr_t);
 extern void drawarea(xcWidget, caddr_t, caddr_t);
 extern void standard_element_delete(short);
@@ -220,9 +233,9 @@ extern int fcompare(const void *, const void *);
 extern void dragfilebox(xcWidget, caddr_t, XMotionEvent *);
 extern void startfiletrack(xcWidget, caddr_t, XCrossingEvent *);
 extern void endfiletrack(xcWidget, caddr_t, XCrossingEvent *);
-extern char *getcrashfilename();
-extern void crashrecover();
-extern void findcrashfiles();
+extern char *getcrashfilename(void);
+extern void crashrecover(void);
+extern void findcrashfiles(void);
 extern void listfiles(xcWidget, popupstruct *, caddr_t);
 extern void newfilelist(xcWidget, popupstruct *);
 extern void fileselect(xcWidget, popupstruct *, XButtonEvent *);
@@ -234,7 +247,7 @@ extern int lookdirectory(char *, int);
 /* from files.c: */
 
 #ifdef ASG
-extern void importspice();
+extern void importspice(void);
 #endif
 
 extern char *ridnewline(char *);
@@ -254,17 +267,17 @@ extern void varcheck(FILE *, short, objectptr, short *, genericptr, u_char);
 extern void varfcheck(FILE *, float, objectptr, short *, genericptr, u_char);
 extern void varpcheck(FILE *, short, objectptr, int, short *, genericptr, u_char);
 extern void getfile(xcWidget, pointertype, caddr_t);
-extern Boolean nextfilename();
-extern void loadfontlib();
+extern Boolean nextfilename(void);
+extern void loadfontlib(void);
 extern void loadglib(Boolean, short, short);
-extern void loadulib();
-extern void loadblib();
+extern void loadulib(void);
+extern void loadblib(void);
 extern void getlib(xcWidget, caddr_t, caddr_t);
 extern void getuserlib(xcWidget, caddr_t, caddr_t);
 extern Boolean loadlibrary(short);
 extern void startloadfile(int);
-extern void normalloadfile();
-extern void importfile();
+extern void normalloadfile(void);
+extern void importfile(void);
 extern Boolean loadfile(short, int);
 extern void readlabel(objectptr, char *, stringpart **);
 extern void readparams(objectptr, objinstptr, objectptr, char *);
@@ -274,23 +287,34 @@ extern char *varpscan(objectptr, char *, short *, genericptr, int, int, u_char);
 extern char *varscan(objectptr, char *, short *, genericptr, u_char);
 extern char *varfscan(objectptr, char *, float *, genericptr, u_char);
 extern objinstptr addtoinstlist(int, objectptr, Boolean);
-extern Boolean objectread(FILE *, objectptr, short, short, short, char *, int);
+extern Boolean objectread(FILE *, objectptr, short, short, short, char *,
+		int, TechPtr);
 void importfromlibrary(short, char *, char *);
-objectptr *new_library_object(short, char *, objlistptr *);
+objectptr *new_library_object(short, char *, objlistptr *, TechPtr);
 Boolean library_object_unique(short, objectptr, objlistptr);
 void add_object_to_library(short, objectptr);
 u_char *find_delimiter(u_char *);
 char standard_delimiter_end(char);
 
+extern Boolean CompareTechnology(objectptr, char *);
+extern TechPtr LookupTechnology(char *);
+extern TechPtr GetObjectTechnology(objectptr);
+extern TechPtr AddNewTechnology(char *, char *);
+extern void AddObjectTechnology(objectptr);
+
 #ifdef TCL_WRAPPER
-void setfile(char *, int);
-extern xcTimeOutProc savetemp(caddr_t);
+extern void setfile(char *, int);
+extern void savetemp(ClientData);
 #else
 extern void setfile(xcWidget, xcWidget, caddr_t);
-extern xcTimeOutProc savetemp(caddr_t, xcIntervalId *);
+extern void savetemp(XtPointer, xcIntervalId *);
 #endif
-extern void savelibpopup(xcWidget, pointertype, caddr_t);
-extern void savelibrary(xcWidget, int);
+extern void incr_changes(objectptr);
+extern void savelibpopup(xcWidget, char *, caddr_t);
+#ifndef TCL_WRAPPER
+extern void savelibrary(xcWidget, char *);
+#endif
+extern void savetechnology(char *, char *);
 extern void findfonts(objectptr, short *);
 extern void savefile(short);
 extern int printRGBvalues(char *, int, const char *);
@@ -323,7 +347,8 @@ extern float fsqwirelen(XfPoint *, XfPoint *);
 extern int wirelength(XPoint *, XPoint *);
 extern long finddist(XPoint *,XPoint *, XPoint *);
 extern void calcarc(arcptr);
-extern void initsplines();
+extern void decomposearc(pathptr);
+extern void initsplines(void);
 extern void computecoeffs(splineptr, float *, float *, float *, float *,
                           float *, float *);
 extern void calcspline(splineptr);
@@ -333,13 +358,19 @@ extern float findsplinemin(splineptr, XPoint *);
 extern short closepoint(polyptr, XPoint *);
 extern short closedistance(polyptr, XPoint *);
 extern void updateinstparam(objectptr);
-extern short checkbounds();
+extern short checkbounds(void);
 extern void window_to_user(short, short, XPoint *);
 extern void user_to_window(XPoint, XPoint *);
-extern float UTopScale();
+
+extern float UTopScale(void);
 extern float UTopTransScale(float);
-extern XPoint UGetCursor();
-extern XPoint UGetCursorPos();
+extern float UTopDrawingScale(void);
+extern int UTopRotation(void);
+extern void UTopOffset(int *, int *);
+extern void UTopDrawingOffset(int *, int *);
+
+extern XPoint UGetCursor(void);
+extern XPoint UGetCursorPos(void);
 extern void u2u_snap(XPoint *);
 extern void snap(short, short, XPoint *);
 extern void UResetCTM(Matrix *);
@@ -351,6 +382,7 @@ extern void USlantCTM(Matrix *, float);
 extern void UPreScaleCTM(Matrix *);
 extern short flipadjust(short);
 extern void UPreMultCTM(Matrix *, XPoint, float, short);
+extern void UPreMultCTMbyMat(Matrix *, Matrix *);
 extern void UTransformbyCTM(Matrix *, XPoint *, XPoint *, short);
 extern void UfTransformbyCTM(Matrix *, XfPoint *, XPoint *, short);
 extern void UPopCTM(void);
@@ -359,19 +391,22 @@ extern void UTransformPoints(XPoint *, XPoint *, short, XPoint, float,
                              short);
 extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
                              short);
-extern void manhattanize(XPoint *, polyptr);
+extern void manhattanize(XPoint *, polyptr, short, Boolean);
 extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
 extern void objinstbbox(objinstptr, XPoint *, Boolean);
 extern void labelbbox(labelptr, XPoint *, objinstptr);
+extern void graphicbbox(graphicptr, XPoint *);
+extern void calcinstbbox(genericptr *, short *, short *, short *, short *);
 extern void calcbboxsingle(genericptr *, objinstptr, short *, short *, short *, short *);
 extern Boolean object_in_library(short, objectptr);
+extern void calcbboxinst(objinstptr);
 extern short find_object(objectptr, objectptr);
 extern void updatepagebounds(objectptr);
 extern void calcbbox(objinstptr);
 extern void calcbboxparam(objectptr, int);
 extern void singlebbox(genericptr *);
-extern void calcbboxselect();
+extern void calcbboxselect(void);
 extern void calcbboxvalues(objinstptr, genericptr *);
 extern void centerview(objinstptr);
 extern void refresh(xcWidget, caddr_t, caddr_t);
@@ -389,7 +424,8 @@ extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
 extern void UDrawXLine(XPoint, XPoint);
 extern void UDrawBox(XPoint, XPoint);
-extern void UDrawBBox();
+extern void UDrawRescaleBox(XPoint *);
+extern void UDrawBBox(void);
 extern void strokepath(XPoint *, short, short, float);
 extern void makesplinepath(splineptr, XPoint *);
 extern void UDrawSpline(splineptr);
@@ -398,12 +434,12 @@ extern void UDrawPolygon(polyptr);
 extern void UDrawArc(arcptr);
 extern void UDrawPath(pathptr);
 extern void UDrawObject(objinstptr, short, int, pushlistptr *);
-extern void TopDoLatex();
+extern void TopDoLatex(void);
 
 /* from help.c: */
 
 extern void showhsb(xcWidget, caddr_t, caddr_t);
-extern void printhelppix();
+extern void printhelppix(void);
 extern void starthelp(xcWidget, caddr_t, caddr_t);
 extern void simplescroll(xcWidget, xcWidget, XPointerMovedEvent *);
 extern void exposehelp(xcWidget, caddr_t, caddr_t);
@@ -421,11 +457,11 @@ extern char *key_binding_to_string(xcWidget, int);
 extern char *compat_key_to_string(xcWidget, int);
 extern char *func_to_string(int);
 extern char *key_to_string(int);
-extern void printeditbindings();
+extern void printeditbindings(void);
 extern int add_vbinding(xcWidget, int, int, short);
 extern int add_binding(xcWidget, int, int);
 extern int add_keybinding(xcWidget, const char *, const char *);
-extern void default_keybindings();
+extern void default_keybindings(void);
 extern int remove_binding(xcWidget, int, int);
 extern void remove_keybinding(xcWidget, const char *, const char *);
 
@@ -436,17 +472,17 @@ extern void mode_tempbinding(int, int);
 
 /* from libraries.c: */
 
-extern short findhelvetica();
-extern void catreturn();
+extern short findhelvetica(void);
+extern void catreturn(void);
 extern int pageposition(short, int, int, int);
-extern short pagelinks();
+extern short pagelinks(int);
 extern short *pagetotals(int, short);
 extern Boolean is_virtual(objinstptr);
 extern int is_page(objectptr);
 extern int is_library(objectptr);
-extern int name_to_library(char *);
-extern void lib_set_changes();
-extern void lib_mark_changed(int);
+extern int NameToLibrary(char *);
+extern void tech_set_changes(TechPtr);
+extern void tech_mark_changed(TechPtr);
 extern int libfindobject(objectptr, int *);
 extern int libmoveobject(objectptr, int);
 extern void pagecat_op(int, int, int);
@@ -458,13 +494,13 @@ extern void updatepagelib(short, short);
 extern void pagecatmove(int, int);
 extern void composelib(short);
 extern short finddepend(objinstptr, objectptr **);
-extern void cathide();
-extern void catvirtualcopy();
-extern void catdelete();
+extern void cathide(void);
+extern void catvirtualcopy(void);
+extern void catdelete(void);
 extern void catmove(int, int);
-extern void copycat();
+extern void copycat(void);
 extern void catalog_op(int, int, int);
-extern void changecat();
+extern void changecat(void);
 extern void startcatalog(xcWidget, pointertype, caddr_t);
 
 /* from menucalls.c: */
@@ -480,6 +516,12 @@ extern void setkern(xcWidget, stringpart *);
 extern void setdscale(xcWidget, XPoint *);
 extern void setosize(xcWidget, objinstptr);
 extern void setwwidth(xcWidget, void *);
+#ifdef TCL_WRAPPER
+extern void renamepage(short);
+extern void renamelib(short);
+extern void setcolormark(int);
+extern void setallstylemarks(u_short);
+#endif
 extern labelptr gettextsize(float **);
 extern void stringparam(xcWidget, caddr_t, caddr_t);
 extern int setelementstyle(xcWidget, u_short, u_short);
@@ -489,15 +531,16 @@ extern void setcolorscheme(Boolean);
 extern void getgridtype(xcWidget, pointertype, caddr_t);
 extern void newlibrary(xcWidget, caddr_t, caddr_t);
 extern int createlibrary(Boolean);
-extern void makepagebutton();
-extern int findemptylib();
+extern void makepagebutton(void);
+extern int findemptylib(void);
 extern polyptr checkforbbox(objectptr);
 #ifdef TCL_WRAPPER
 extern void setcolor(xcWidget, int);
+extern void setfontmarks(short, short);
 #endif
 extern void startparam(xcWidget, pointertype, caddr_t);
 extern void startunparam(xcWidget, pointertype, caddr_t);
-extern void setdefaultfontmarks();
+extern void setdefaultfontmarks(void);
 extern void setjustbit(xcWidget, pointertype, caddr_t);
 extern void setpinjustbit(xcWidget, pointertype, caddr_t);
 extern void setjust(xcWidget, pointertype, labelptr, short);
@@ -517,6 +560,7 @@ extern Boolean dospecial(void);
 
 /* from xtfuncs.c: */
 
+extern void makenewfontbutton(void);  /* either here or menucalls.c */
 #ifndef TCL_WRAPPER
 extern void setfloat(xcWidget, float *);
 extern void autoset(xcWidget, xcWidgetList, caddr_t);
@@ -559,7 +603,7 @@ extern void getwwidth(xcWidget, caddr_t, caddr_t);
 extern void getfloat(xcWidget, float *, caddr_t);
 extern void setfilename(xcWidget, char **);
 extern void setpagelabel(xcWidget, char *);
-extern void makenewfontbutton();
+extern void makenewfontbutton(void);
 extern void newpagemenu(xcWidget, pointertype, caddr_t);
 extern void makenewencodingbutton(char *, char);
 extern void toggle(xcWidget, pointertype, Boolean *);
@@ -585,19 +629,21 @@ extern void addnewfont(xcWidget, caddr_t, caddr_t);
 #ifdef TCL_WRAPPER
 extern Tcl_Obj *tclglobals(objinstptr);
 extern Tcl_Obj *tcltoplevel(objinstptr);
+void ratsnest(objectptr);
 #endif
 
+extern int NameToPinLocation(objinstptr, char *, int *, int *);
 extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
 extern void NameToPosition(objinstptr, labelptr, XPoint *);
 extern XPoint *NetToPosition(int, objectptr);
+extern int getsubnet(int, objectptr);
 extern void invalidate_netlist(objectptr);
 extern void remove_netlist_element(objectptr, genericptr);
 extern int updatenets(objinstptr, Boolean);
 extern void createnets(objinstptr, Boolean);
-extern void destroynets();
 extern Boolean nonnetwork(polyptr);
-extern int globalmax();
+extern int globalmax(void);
 extern LabellistPtr geninfolist(objectptr, objinstptr, char *);
 extern void gennetlist(objinstptr);
 extern void gencalls(objectptr);
@@ -609,7 +655,9 @@ extern void resolve_devindex(objectptr, Boolean);
 extern void copy_bus(Genericlist *, Genericlist *);
 extern Genericlist *is_resolved(genericptr *, pushlistptr, objectptr *);
 extern Boolean highlightnet(objectptr, objinstptr, int, u_char);
+extern void highlightnetlist(objectptr, objinstptr, u_char);
 extern int pushnetwork(pushlistptr, objectptr);
+extern Boolean match_buses(Genericlist *, Genericlist *, int);
 extern int onsegment(XPoint *, XPoint *, XPoint *);
 extern Boolean neardist(long);
 extern Boolean nearpoint(XPoint *, XPoint *);
@@ -620,6 +668,7 @@ extern long zsign(long, long);
 extern Boolean mergenets(objectptr, Genericlist *, Genericlist *);
 extern void removecall(objectptr, CalllistPtr);
 extern Genericlist *addpin(objectptr, objinstptr, labelptr, Genericlist *);
+extern Genericlist *addglobalpin(objectptr, objinstptr, labelptr, Genericlist *);
 extern void addcall(objectptr, objectptr, objinstptr);
 extern void addport(objectptr, Genericlist *);
 extern Boolean addportcall(objectptr, Genericlist *, Genericlist *);
@@ -647,18 +696,22 @@ extern void outputpcb(struct Ptab *, FILE *);
 extern void freepcb(struct Ptab *);
 extern void freegenlist(Genericlist *);
 extern void freepolylist(PolylistPtr *);
+extern void freenetlist(objectptr);
 extern void freelabellist(LabellistPtr *);
 extern void freecalls(CalllistPtr);
 extern void freenets(objectptr);
 extern void freetemplabels(objectptr);
-extern void freeglobals();
+extern void freeglobals(void);
 extern void destroynets(objectptr);
 extern int  cleartraversed(objectptr);
 extern int  checkvalid(objectptr);
 extern void clearlocalpins(objectptr);
 extern void append_included(char *);
 extern Boolean check_included(char *);
-extern void free_included();
+extern void free_included(void);
+
+/* from ngspice.c: */
+extern int exit_spice(void);
 
 /* from parameter.c: */ 
 
@@ -667,39 +720,44 @@ extern oparamptr match_param(objectptr, char *);
 extern oparamptr match_instance_param(objinstptr, char *);
 extern oparamptr find_param(objinstptr, char *);
 extern int get_num_params(objectptr);
+extern void free_all_eparams(genericptr);
 extern void free_object_param(objectptr, oparamptr);
-extern void free_instance_param(objinstptr, oparamptr);
+extern oparamptr free_instance_param(objinstptr, oparamptr);
 extern void free_element_param(genericptr, eparamptr);
 
 extern oparamptr make_new_parameter(char *);
 extern eparamptr make_new_eparam(char *);
 
-extern void makeexprparam(objectptr, char *, char *, int);
-extern void makefloatparam(objectptr, char *, float);
-extern void makestringparam(objectptr, char *, stringpart *);
+extern char *getnumericalpkey(u_int);
+extern char *makeexprparam(objectptr, char *, char *, int);
+extern Boolean makefloatparam(objectptr, char *, float);
+extern Boolean makestringparam(objectptr, char *, stringpart *);
+extern void std_eparam(genericptr, char *);
 extern void indicateparams(genericptr);
-extern void makenumericalp(genericptr *, u_int, char *);
+extern void setparammarks(genericptr);
+extern void makenumericalp(genericptr *, u_int, char *, short);
 extern void noparmstrcpy(u_char *, u_char *);
-extern void insertparam();
+extern void insertparam(void);
 extern void makeparam(labelptr, char *);
 extern void searchinst(objectptr, objectptr, char *);
 extern stringpart *searchparam(stringpart *);
 extern void unmakeparam(labelptr, stringpart *);
-extern void unmakenumericalp(genericptr *, u_int);         
+extern void removenumericalp(genericptr *, u_int);         
 extern void unparameterize(int);
-extern void parameterize(int, char *);
+extern void parameterize(int, char *, short);
 extern genericptr findparam(objectptr, void *, u_char);
 extern Boolean paramcross(objectptr, labelptr);
 extern oparamptr parampos(objectptr, labelptr, char *, short *, short *);
 extern int opsubstitute(objectptr, objinstptr);
 extern int psubstitute(objinstptr);
+extern Boolean has_param(genericptr);
 extern oparamptr copyparameter(oparamptr);
 extern void pwriteback(objinstptr);
 extern short paramlen(u_char *);
 extern int natstrlen(u_char *);
 extern int natstrcmp(u_char *, u_char *);
 extern void curtail(u_char *);
-extern int checklibtop();
+extern int checklibtop(void);
 extern void removeinst(objinstptr);
 extern void removeparams(objectptr);
 extern void resolveparams(objinstptr);
@@ -707,8 +765,9 @@ extern void resolveparams(objinstptr);
 /* from python.c: */
 
 #ifdef HAVE_PYTHON
-extern void init_interpreter();
-extern void exit_interpreter();
+extern int python_key_command(int);
+extern void init_interpreter(void);
+extern void exit_interpreter(void);
 #endif
 
 #ifdef HAVE_XPM
@@ -718,9 +777,9 @@ extern xcWidget *pytoolbuttons(int *);
 /* from rcfile.c: */
 
 extern short execcommand(short, char *);
-extern void defaultscript();
-extern void execscript();
-extern void loadrcfile();
+extern void defaultscript(void);
+extern void execscript(void);
+extern void loadrcfile(void);
 #ifndef HAVE_PYTHON
 extern short readcommand(short, FILE *);
 #endif
@@ -737,6 +796,8 @@ extern short *collect_graphics(short *);
 /* from flate.c */
 
 #ifdef HAVE_LIBZ
+extern u_long large_deflate(u_char *, u_long, u_char *, u_long);
+extern u_long large_inflate(u_char *, u_long, u_char **, u_long);
 extern unsigned long ps_deflate (unsigned char *, unsigned long,
 	unsigned char *, unsigned long);
 extern unsigned long ps_inflate (unsigned char *, unsigned long,
@@ -745,38 +806,39 @@ extern unsigned long ps_inflate (unsigned char *, unsigned long,
 
 /* from render.c: */
 
-extern void ghostinit();
+extern void ghostinit(void);
 extern void send_client(Atom);
-extern void ask_for_next();
-extern void start_gs();
+extern void ask_for_next(void);
+extern void start_gs(void);
 extern void parse_bg(FILE *, FILE *);
-extern void bg_get_bbox();
+extern void bg_get_bbox(void);
 extern void backgroundbbox(int);
 extern void readbackground(FILE *);
 extern void savebackground(FILE *, char *);
 extern void register_bg(char *);
-extern void loadbackground();
+extern void loadbackground(void);
 extern void send_to_gs(char *);
-extern int renderbackground();
-extern int copybackground();
-extern int exit_gs();
-extern int reset_gs();
+extern int renderbackground(void);
+extern int copybackground(void);
+extern int exit_gs(void);
+extern int reset_gs(void);
 
 #ifndef TCL_WRAPPER
-extern Boolean render_client();
+extern Boolean render_client(XEvent *);
 #endif
 
 /* from schema.c: */
 
 extern objectptr NameToPageObject(char *, objinstptr *, int *);
 extern objectptr NameToObject(char *, objinstptr *, Boolean);
+extern int checkpagename(objectptr);
 extern void callwritenet(xcWidget, pointertype, caddr_t);
 extern void startconnect(xcWidget, caddr_t, caddr_t);
 extern void connectivity(xcWidget, caddr_t, caddr_t);
 extern Boolean setobjecttype(objectptr);
 extern void pinconvert(labelptr, pointertype);
 extern void dopintype(xcWidget, pointertype, caddr_t);
-extern void setsymschem();
+extern void setsymschem(void);
 extern int findpageobj(objectptr);
 extern void collectsubschems(int);
 extern int findsubschems(int, objectptr, int, short *);
@@ -786,7 +848,7 @@ extern int checksym(objectptr, char *);
 extern void changeotherpins(labelptr, stringpart *);
 extern void swapschem(int, int);
 extern void dobeforeswap(xcWidget, caddr_t, caddr_t);
-extern void schemdisassoc();
+extern void schemdisassoc(void);
 extern void startschemassoc(xcWidget, pointertype, caddr_t);
 extern Boolean schemassoc(objectptr, objectptr);
 #ifndef TCL_WRAPPER
@@ -802,27 +864,32 @@ extern Boolean checkselect(short);
 extern void geneasydraw(short, int, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
 extern selection *genselectelement(short, u_char, objectptr, objinstptr);
-extern short *allocselect();
-extern void setoptionmenu();
+extern short *allocselect(void);
+extern void setoptionmenu(void);
 extern int test_insideness(int, int, XPoint *);
 extern Boolean pathselect(genericptr *, short, float);
 extern Boolean areaelement(genericptr *);
-extern void selectarea();
+extern void selectarea(void);
 extern void startdesel(xcWidget, caddr_t, caddr_t);
 extern void deselect(xcWidget, caddr_t, caddr_t);
-extern void draw_normal_selected();
-extern void draw_all_selected();
-extern void clearselects_noundo();
-extern void clearselects();
-extern void unselect_all();
+extern void draw_normal_selected(objectptr, objinstptr);
+extern void freeselects(void);
+extern void draw_all_selected(void);
+extern void clearselects_noundo(void);
+extern void clearselects(void);
+extern void unselect_all(void);
+extern void select_connected_pins();
+extern void reset_connected_pins();
 extern selection *recurselect(short, u_char, pushlistptr *);
 extern short *recurse_select_element(short, u_char);
-extern void startselect();
-extern void trackselarea();
+extern void startselect(void);
+extern void trackselarea(void);
+extern void trackrescale(void);
 
 /* from text.c: */
 
 extern Boolean hasparameter(labelptr);
+extern void joinlabels(void);
 extern void drawparamlabels(labelptr, short);
 extern stringpart *nextstringpart(stringpart *, objinstptr);
 extern stringpart *nextstringpartrecompute(stringpart *, objinstptr);
@@ -841,6 +908,8 @@ extern char *textprintnet(char *, char *, Genericlist *);
 extern int textcomp(stringpart *, char *, objinstptr);
 extern int textncomp(stringpart *, char *, objinstptr);
 extern int stringcomp(stringpart *, stringpart *);
+extern Boolean issymbolfont(int);
+extern Boolean isisolatin1(int);
 extern int stringcomprelaxed(stringpart *, stringpart *, objinstptr);
 extern int stringparts(stringpart *);
 extern int stringlength(stringpart *, Boolean, objinstptr);
@@ -856,7 +925,7 @@ extern int find_ordinal(int, labelptr, objinstptr);
 
 extern short UDrawChar(u_char, short, short, int, int);
 extern void UDrawString(labelptr, int, objinstptr);
-extern TextExtents ULength(stringpart *, objinstptr, float, short, XPoint *);
+extern TextExtents ULength(labelptr, objinstptr, float, short, XPoint *);
 extern void undrawtext(labelptr);
 extern void redrawtext(labelptr);
 extern void composefontlib(short);
@@ -869,9 +938,11 @@ extern void W1printf(char *, ...);
 extern void W2printf(char *, ...);
 extern void W3printf(char *, ...);
 
-extern XCWindowData *create_new_window();
+extern XCWindowData *create_new_window(void);
+extern void pre_initialize(void);
+extern void post_initialize(void);
 extern void delete_window(XCWindowDataPtr);
-extern void printeventmode();
+extern void printeventmode(void);
 extern void popupprompt(xcWidget, char *, char *, void (*function)(),
                         buttonsave *, const char *);
 extern void getproptext(xcWidget, propstruct *, caddr_t);
@@ -882,7 +953,8 @@ extern int xc_getlayoutcolor(int);
 extern int query_named_color(char *);
 extern caddr_t CvtStringToPixel(XrmValuePtr, int *, XrmValuePtr, XrmValuePtr);
 extern void outputpopup(xcWidget, caddr_t, caddr_t);
-extern void docommand();
+extern void docommand(void);
+extern int  installowncmap(void);  /* sometimes from xtgui.c */
 extern void destroypopup(xcWidget, popupstruct *, caddr_t);
 extern int xc_alloccolor(char *);
 extern void dointr(int);
@@ -891,10 +963,10 @@ extern u_short countchanges(char **);
 extern u_short getchanges(objectptr);
 extern void quitcheck(xcWidget, caddr_t, caddr_t);
 extern void quit(xcWidget, caddr_t);
-extern void resizetoolbar();
+extern void resizetoolbar(void);
 extern void writescalevalues(char *, char *, char *);
 #ifdef TCL_WRAPPER
-extern Tcl_Obj *Tcl_NewHandleObj();
+extern Tcl_Obj *Tcl_NewHandleObj(void *);
 extern int Tcl_GetHandleFromObj(Tcl_Interp *, Tcl_Obj *, void **);
 #else
 extern void updatetext(xcWidget, xcWidgetList, caddr_t);
@@ -902,4 +974,4 @@ extern void delwin(xcWidget, popupstruct *, XClientMessageEvent *);
 #endif
 
 
-extern void makecursors();
+extern void makecursors(void);
