@@ -47,8 +47,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Globaldata xobjs;
-extern Clientdata areastruct;
-extern short	  eventmode;
+extern XCWindowData *areawin;
 extern int	  *appcolors;
 extern xcWidget     menuwidgets[];
 extern xcWidget	  netbutton;
@@ -80,52 +79,11 @@ void callwritenet(xcWidget w, pointertype mode, caddr_t calldata)
 	break;
      case 4:
 	/* Auto-numbering:  no output generated */
-	writenet(topobject, "idxpcb", "");
+	writenet(topobject, "indexpcb", "");
 	break;
    }
 }
 
-/*--------------------------------------------------------------*/
-/* Enable schematic capture (set schemon and realize widgets)	*/
-/* If mode == 1, do a redraw (e.g., not called during a file	*/
-/* load)							*/
-/*--------------------------------------------------------------*/
-
-void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
-{
-#ifndef TCL_WRAPPER
-   Arg wargs[1];
-#endif
-
-   if (areastruct.schemon) {
-      areastruct.schemon = False;
-#ifndef TCL_WRAPPER
-      XtUnmanageChild(wschema);
-      XtUnmanageChild(wsymb);
-      XtUnmanageChild(netbutton);
-
-      XtSetArg(wargs[0], XtNlabel, "Enable XSchema");
-      XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
-#endif
-   }
-   else {
-      areastruct.schemon = True;
-#ifndef TCL_WRAPPER
-      XtManageChild(wschema);
-      XtManageChild(wsymb);
-      XtManageChild(netbutton);
-
-      XtSetArg(wargs[0], XtNlabel, "Disable XSchema");
-      XtSetValues(OptionsEnableXSchemaButton, wargs, 1);
-#endif
-   }
-
-   /* Redraw the screen, since this action may effect the presence of	*/
-   /* pins and schematic informational labels.				*/
-
-   if (mode) drawarea(NULL, NULL, NULL);
-}
-
 /*----------------------------------------------------------------------*/
 /* Find the page object and instance with the indicated name.		*/
 /*----------------------------------------------------------------------*/
@@ -147,16 +105,33 @@ objectptr NameToPageObject(char *objname, objinstptr *ret_inst, int *ret_page)
 
 /*----------------------------------------------------------------------*/
 /* Find the object with the indicated name.				*/
+/*									*/
+/* WARNING:  If no library technology is given, XCircuit will return	*/
+/* the first object with the name "objname".  If there are multiple	*/
+/* objects of the same name in different technologies, then the one	*/
+/* returned may not be the one expected!				*/
 /*----------------------------------------------------------------------*/
 
 objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
 {
    int i;
    liblistptr spec;
+   Boolean notech = FALSE;
+   char *techptr;
+
+   if (strstr(objname, "::") == NULL) notech = TRUE;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
-         if (!strcmp(objname, spec->thisinst->thisobject->name)) {
+	 techptr = spec->thisinst->thisobject->name;
+	 if (notech) {
+	    techptr = strstr(spec->thisinst->thisobject->name, "::");
+	    if (techptr == NULL)
+	       techptr = spec->thisinst->thisobject->name;
+ 	    else
+	       techptr += 2;
+	 }
+         if (!strcmp(objname, techptr)) {
 	    if (ret_inst) *ret_inst = spec->thisinst;
 	    return spec->thisinst->thisobject;
 	 }
@@ -176,7 +151,7 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
 int checkpagename(objectptr thispageobj)
 {
    int p, thispage;
-   char *objname = thispageobj->name;
+   /* char *objname = thispageobj->name; (jdk) */
    Boolean changed;
    Boolean update = False;
    char *clnptr = NULL;
@@ -235,7 +210,7 @@ int checkpagename(objectptr thispageobj)
 
 void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   if (areastruct.selects > 0)
+   if (areawin->selects > 0)
       connectivity(button, clientdata, calldata);
 }
 
@@ -253,22 +228,22 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    int depth, lbus, netid, subnetid;
    buslist *sbus;
    pushlistptr seltop, nextptr;
-   objectptr nettop, thisobj, pschem;
+   objectptr nettop, pschem; /*  thisobj, (jdk) */
    char *snew;
    stringpart *ppin;
 
    /* erase any existing highlighted network */
-   highlightnetlist(topobject, areastruct.topinstance, 0);
+   highlightnetlist(topobject, areawin->topinstance, 0);
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
-   seltop->thisinst = areastruct.topinstance;
+   seltop->thisinst = areawin->topinstance;
    seltop->next = NULL;
 
    /* pick the first selection that looks like a valid network part */
 
-   if (areastruct.selects > 0) {
-      for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++) {
+   if (areawin->selects > 0) {
+      for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++) {
 	 ggen = *(topobject->plist + *gsel);
          if (SELECTTYPE(gsel) == LABEL) {
 	    labelptr glab = SELTOLABEL(gsel);
@@ -280,8 +255,8 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	 }
       }
    }
-   if ((areastruct.selects == 0) || (gsel == areastruct.selectlist
-		+ areastruct.selects)) {
+   if ((areawin->selects == 0) || (gsel == areawin->selectlist
+		+ areawin->selects)) {
       rselect = recurselect(POLYGON | LABEL | OBJINST, MODE_CONNECT, &seltop);
       /* Should look at the top scorer, when scoring has been implemented */
       if (rselect && (rselect->selects > 0)) {
@@ -305,7 +280,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
          destroynets(topobject);
-         createnets(areastruct.topinstance);
+         createnets(areawin->topinstance, FALSE);
       }
       if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
@@ -329,7 +304,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
 	 if (netlist->subnets == 0) {
 	    ppin = nettopin(netlist->net.id, pschem, NULL);
-            snew = textprint(ppin, areastruct.topinstance);
+            snew = textprint(ppin, areawin->topinstance);
 	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
             free(snew);
 	 }
@@ -342,14 +317,14 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	       netid = sbus->netid;
 	       subnetid = sbus->subnetid;
 	       ppin = nettopin(netid, pschem, NULL);
-               snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
+               snew = textprintsubnet(ppin, areawin->topinstance, subnetid);
 	       sprintf(sptr, "%s ", snew);
 	       sptr += strlen(snew) + 1;
 	       free(snew);
 	    }
 	    sprintf(sptr, "in %s", nettop->name);
 	 }
-	 Wprintf(_STR2);
+	 Wprintf("%s", _STR2);
 
 #ifdef TCL_WRAPPER
 	 Tcl_SetObjResult(xcinterp, Tcl_NewStringObj(snew, strlen(snew)));
@@ -385,6 +360,9 @@ Boolean setobjecttype(objectptr cschem)
    genericptr *cgen;
    labelptr clab;
 
+   /* If networks are specifically prohibited. . . */
+   if (cschem->schemtype == NONETWORK) return False;
+
    /* Apply only to schematic objects */
 
    if ((cschem->schemtype != PRIMARY) && (cschem->schemtype != SECONDARY)) {
@@ -444,7 +422,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
    char typestr[40];
    short savetype = -1;
 
-   if (areastruct.selects == 0) {
+   if (areawin->selects == 0) {
       Wprintf("Must first select a label to change type");
       return;
    }
@@ -465,8 +443,8 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
          break;
    }
 
-   for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++)
+   for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++)
       if (SELECTTYPE(gsel) == LABEL) {
 	 labelptr glab = SELTOLABEL(gsel);
 	 savetype = glab->pin;
@@ -477,7 +455,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
    if (savetype >= 0) {
       unselect_all();
       drawarea(NULL, NULL, NULL);
-      Wprintf(typestr);
+      Wprintf("%s", typestr);
    }
    else {
       Wprintf("No labels selected.");
@@ -649,7 +627,7 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 
 void collectsubschems(int toppage)
 {
-   int loctop, level = 0;
+  int loctop; /* , level = 0; (jdk) */
    objectptr cschem;
    short *pagelist, pageno;
    Pagedata *curpage;
@@ -684,6 +662,38 @@ void collectsubschems(int toppage)
    free((char *)pagelist);
 }
 
+/*------------------------------------------------------*/
+/* compare_qualified(str1, str2) ---			*/
+/*							*/
+/* 	String comparison for technology-qualified	*/
+/*	names.  str2" must be a fully-qualified name.	*/
+/*	"str1" may be qualified or unqualified.  If	*/
+/*	unqualified, compare_qualified will return TRUE	*/
+/*	for any "str2" that matches the name part of	*/
+/*	the technology::name format.			*/
+/*							*/
+/*	Return value:  TRUE if match, FALSE if not.	*/
+/*------------------------------------------------------*/
+
+Boolean compare_qualified(char *str1, char *str2)
+{
+   char *sptr1, *sptr2;
+   Boolean qual1, qual2;
+
+   sptr2 = strstr(str2, "::");
+   qual2 = (sptr2 == NULL) ? FALSE : TRUE;
+
+   if (!qual2) return (!strcmp(str1, str2));
+
+   sptr1 = strstr(str1, "::");
+   qual1 = (sptr1 == NULL) ? FALSE : TRUE;
+
+   if (qual1) return (!strcmp(str1, str2));
+
+   sptr2 += 2;
+   return (!strcmp(str1, sptr2));
+}
+
 /*-------------------------------------------------------*/
 /* Check if top-level page is the same name as a library */
 /* object; if so, connect it.				 */
@@ -699,13 +709,13 @@ int checkschem(objectptr thisobj, char *cname)
    objectptr *tlib;
    short i, j;
 
-   if ((areastruct.schemon == False) || (thisobj->symschem != NULL)) return 0;
+   if (thisobj->symschem != NULL) return 0;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
 	 tlib = xobjs.userlibs[i].library + j;
 
-         if (!strcmp(cname, (*tlib)->name)) {
+         if (compare_qualified(cname, (*tlib)->name)) {
 	    thisobj->symschem = (*tlib);
 	    thisobj->schemtype = PRIMARY;
 	    (*tlib)->symschem = thisobj;
@@ -727,12 +737,12 @@ int checksym(objectptr symobj, char *cname)
    short cpage;
    objectptr checkpage;
 
-   if (areastruct.schemon == False || symobj->symschem != NULL) return 0;
+   if (symobj->symschem != NULL) return 0;
 
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       if (xobjs.pagelist[cpage]->pageinst != NULL) {
          checkpage = xobjs.pagelist[cpage]->pageinst->thisobject;
-         if (!strcmp(checkpage->name, cname)) {
+         if (compare_qualified(cname, checkpage->name)) {
 	    symobj->symschem = checkpage;
 	    symobj->schemtype = SYMBOL;
 	    checkpage->symschem = symobj;
@@ -796,7 +806,6 @@ void swapschem(int allow_create, int libnum)
    pushlistptr stacktop;
    short loclibnum = libnum;
 
-   if (areastruct.schemon == False) return;
    if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
    if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
@@ -817,11 +826,11 @@ void swapschem(int allow_create, int libnum)
 	 /* Push the current instance onto the push stack */
 	 /* Change the page without destroying the pushlist */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 stacktop = areastruct.stack;
-	 areastruct.stack = NULL;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 stacktop = areawin->stack;
+	 areawin->stack = NULL;
          changepage(tpage);
-	 areastruct.stack = stacktop;
+	 areawin->stack = stacktop;
       }
       else {
 	 objectptr *newobject;
@@ -843,11 +852,13 @@ void swapschem(int allow_create, int libnum)
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
-	 calcbbox(areastruct.topinstance);
+	 calcbbox(areawin->topinstance);
       }
 
       /* set links between the two objects */
@@ -886,7 +897,7 @@ void swapschem(int allow_create, int libnum)
 	       (*pinlab)->color = lpin->color;
 	       (*pinlab)->rotation = 0;
 	       (*pinlab)->scale = 1.0;
-	       (*pinlab)->justify = areastruct.justify; 
+	       (*pinlab)->justify = areawin->justify; 
 	       (*pinlab)->position.x = 0;
 	       (*pinlab)->position.y = topobject->parts * (TEXTHEIGHT + 10);
 	       (*pinlab)->passed = NULL;
@@ -897,7 +908,7 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
       }
-      calcbbox(areastruct.topinstance);
+      calcbbox(areawin->topinstance);
 
       /* Recreate the user library with the new symbol */
       if (savepage->schemtype != SYMBOL) composelib(loclibnum + LIBRARY);
@@ -907,10 +918,12 @@ void swapschem(int allow_create, int libnum)
       /* If symschem matches the last entry on the push stack, then we	*/
       /* pop; otherwise, we push.					*/
 
-      if (areastruct.stack && areastruct.stack->thisinst->thisobject
+      if (areawin->stack && areawin->stack->thisinst->thisobject
 			== topobject->symschem) {
-	 areastruct.topinstance = areastruct.stack->thisinst;
-	 pop_stack(&areastruct.stack);
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 areawin->topinstance = areawin->stack->thisinst;
+	 pop_stack(&areawin->stack);
       }
       else {
 	 int p;
@@ -946,8 +959,10 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
 	       
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = syminst;
+	 topobject->viewscale = areawin->vscale;
+	 topobject->pcorner = areawin->pcorner;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = syminst;
       }
    }
 
@@ -985,10 +1000,6 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
    buttonsave *popdata = (buttonsave *)malloc(sizeof(buttonsave));
 
-   /* Check if this function is valid */
-
-   if (areastruct.schemon == False) return;
-
    /* Check for requirement to change the name before creating the symbol */
 
    if ((topobject->symschem == NULL) && (topobject->schemtype == PRIMARY)
