@@ -52,12 +52,31 @@ typedef struct {
    short yaxis;
 } arcinfo;
 
+/* Smaller data structure used when saving paths in the undo stacks.    */
+/* We don't need the rendering data from points[].                      */
+
 typedef struct {
+   u_short	type;
+   int		color;
+   eparamptr	passed;
+   short	cycle;
+   u_short	style;
+   float	width;
+   XPoint	ctrl[4];
+} splineinfo;
+
+typedef struct {
+   int number;
+   pointlist points;
+} pathinfo;
+
+typedef struct editelem {
    genericptr element;		/* element being edited */
    union {
       stringpart *string;	/* original contents of string, for label */
       pointlist points;		/* original polygon */
       arcinfo *arcspecs;	/* original arc values */
+      pathinfo *pathspecs;	/* original parts of a path */
    } save;
 } editelement;
 
@@ -158,7 +177,7 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
 	 k++;
       }
       else
-	 Fprintf(stderr, "Error: element 0x%x in select list but not object\n",
+	 Fprintf(stderr, "Error: element %p in select list but not object\n",
 		egen);
    } 
 
@@ -183,8 +202,12 @@ short *regen_selection(objinstptr thisinst, uselection *srec)
 /*   Free memory allocated to an undo record edit element structure.	*/
 /*----------------------------------------------------------------------*/
 
-void free_editelement(editelement *erec)
+void free_editelement(Undoptr thisrecord)
 {
+   editelement *erec;
+   pathinfo *ppi;
+
+   erec = (editelement *)thisrecord->undodata;
    switch (erec->element->type) {
       case LABEL:
 	 freelabel(erec->save.string);
@@ -195,6 +218,12 @@ void free_editelement(editelement *erec)
       case ARC:
 	 free(erec->save.arcspecs);
 	 break;
+      case PATH:
+	 for (ppi = erec->save.pathspecs; ppi < erec->save.pathspecs +
+		thisrecord->idata; ppi++)
+	    free(ppi->points);
+	 free(erec->save.pathspecs);
+	 break;
    }
    free(erec);
 }
@@ -241,6 +270,7 @@ stringpart *get_original_string(labelptr thislab)
 	    return NULL;
       }
    }
+   return NULL;  /* yes, this can be reached, if thisrecord->next == NULL */
 }
 
 /*----------------------------------------------------------------------*/
@@ -348,7 +378,7 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 {
    va_list args;
-   int usize, drawmode, nval, oval, *idata, snum, deltax, deltay, dir;
+   int drawmode, nval, oval, *idata, snum, deltax, deltay, dir, i;
    short *slist;
    objectptr delobj;
    objinstptr newinst;
@@ -356,7 +386,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    uselection *srec;
    editelement *erec;
    scaleinfo *escale;
-   genericptr egen, newgen;
+   genericptr egen, *ggen, *cgen;
    XPoint *fpoint;
    double scale;
 
@@ -489,6 +519,27 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
 	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
 	       break;
+	    case PATH:
+	       newrecord->idata = ((pathptr)egen)->parts;  /* not needed? */
+	       erec->save.pathspecs = (pathinfo *)malloc(newrecord->idata *
+			sizeof(pathinfo));
+	       for (i = 0; i < newrecord->idata; i++) {
+		  pathinfo *ppi = erec->save.pathspecs + i;
+		  genericptr *pgen = ((pathptr)egen)->plist + i;
+		  switch (ELEMENTTYPE(*pgen)) {
+		     case POLYGON:
+			ppi->number = (TOPOLY(pgen))->number;
+			ppi->points = copypoints((TOPOLY(pgen))->points,
+				ppi->number);
+			break;
+		     case SPLINE:
+			ppi->number = 4;
+			ppi->points = copypoints((pointlist)
+				(TOSPLINE(pgen))->ctrl, 4);
+			break;
+		  }
+	       }
+	       break;
 	 }
 	 newrecord->undodata = (char *)erec;
 	 break;
@@ -564,7 +615,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 
 short undo_one_action()
 {
-   Undoptr thisrecord, chkrecord;
+  Undoptr thisrecord; /* , chkrecord; (jdk) */
    objectptr thisobj;
    objinstptr thisinst;
    uselection *srec;
@@ -572,7 +623,8 @@ short undo_one_action()
    scaleinfo *escale;
    short *slist;
    XPoint *delta, position;
-   int i, snum, npage, opage;
+   int i, j, snum;
+   int savemode;
    float fnum;
    genericptr egen;
    labelptr thislabel;
@@ -588,7 +640,7 @@ short undo_one_action()
    thisrecord = xobjs.undostack;
    if (thisrecord == NULL) {
       Fprintf(stderr, "Nothing to undo!\n");
-      return;
+      return 0;
    }
 
    xobjs.undostack = thisrecord->next;
@@ -604,6 +656,7 @@ short undo_one_action()
    /* Setting eventmode to UNDO_MODE prevents register_for_undo() from 	*/
    /* being called again while executing the event.			*/
 
+   savemode = eventmode;
    eventmode = UNDO_MODE;
 
    /* type-dependent part */
@@ -638,8 +691,10 @@ short undo_one_action()
 	    thisrecord->undodata = NULL;
 	    break;
 	 }
-	 else
+	 else {
+	    /* XXX egen is uninitialized! */
 	    delete_one_element(thisrecord->thisinst, egen);
+	 }
 
 	 /* Put back all the parts */
 	 areawin->selects = thisobj->parts;
@@ -714,6 +769,21 @@ short undo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areawin->area, NULL, NULL);
+	    }
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -737,19 +807,36 @@ short undo_one_action()
 	       calcspline(espline);
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
-	    case ARC: {
-	       arcinfo tmpinfo;
-	       arcptr earc = (arcptr)(erec->element);
-	       tmpinfo.angle1 = earc->angle1; 
-	       tmpinfo.angle2 = earc->angle2; 
-	       tmpinfo.radius = earc->radius; 
-	       tmpinfo.yaxis = earc->yaxis; 
-	       earc->angle1 = erec->save.arcspecs->angle1;
-	       earc->angle2 = erec->save.arcspecs->angle2;
-	       earc->radius = erec->save.arcspecs->radius;
-	       earc->yaxis = erec->save.arcspecs->yaxis;
-	       *(erec->save.arcspecs) = tmpinfo;
-	       calcarc(earc);
+	    case PATH: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly;
+	       splineptr espline;
+	       pathinfo *tmpinfo = erec->save.pathspecs;
+	       pathptr epath = (pathptr)(erec->element);
+	       for (i = 0; i < epath->parts; i++) {
+		  genericptr ggen = *(epath->plist + i);
+		  switch (ELEMENTTYPE(ggen)) {
+		     case POLYGON:
+			epoly = (polyptr)ggen;
+		        tmppts = epoly->points;
+			tmpnum = epoly->number;
+			epoly->points = (erec->save.pathspecs + i)->points;
+			epoly->number = (erec->save.pathspecs + i)->number;
+			(erec->save.pathspecs + i)->points = tmppts;
+			(erec->save.pathspecs + i)->number = tmpnum;
+			break;
+		     case SPLINE:
+			espline = (splineptr)ggen;
+			tmppts = copypoints((pointlist)espline->ctrl, 4);
+			for (j = 0; j < 4; j++)
+			   espline->ctrl[j] = *((erec->save.pathspecs + i)->points + j);
+			free((erec->save.pathspecs + i)->points);
+		        (erec->save.pathspecs + i)->points = tmppts;
+			calcspline(espline);
+			break;
+		  }
+	       }
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -863,7 +950,9 @@ short undo_one_action()
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
+	 select_connected_pins();
 	 placeselects(-(delta->x), -(delta->y), NULL);
+	 reset_connected_pins();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -872,7 +961,16 @@ short undo_one_action()
 	 break;
    }
 
-   eventmode = NORMAL_MODE;	/* Set on a per-event-type basis? */
+   /* Does this need to be set on a per-event-type basis? */
+   switch (savemode) {
+      case CATALOG_MODE:
+      case CATTEXT_MODE:
+	 eventmode = CATALOG_MODE;
+	 break;
+      default:
+	 eventmode = NORMAL_MODE;
+	 break;
+   }
 
    /* Diagnostic, to check if all multiple-event undo series are resolved */
    if (thisrecord->idx < 0) {
@@ -910,7 +1008,8 @@ short redo_one_action()
    uselection *srec;
    editelement *erec;
    scaleinfo *escale;
-   int i, tpage, snum;
+   int i, j, snum;
+   int savemode;
    float fnum;
    labelptr thislabel;
    arcptr thisarc;
@@ -926,7 +1025,7 @@ short redo_one_action()
    thisrecord = xobjs.redostack;
    if (thisrecord == NULL) {
       Fprintf(stderr, "Nothing to redo!\n");
-      return;
+      return 0;
    }
    xobjs.undostack = thisrecord;
    xobjs.redostack = thisrecord->last;
@@ -938,6 +1037,7 @@ short redo_one_action()
       return 0;
    }
 
+   savemode = eventmode;
    eventmode = UNDO_MODE;
 
    /* type-dependent part */
@@ -1040,6 +1140,21 @@ short redo_one_action()
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areawin->area, NULL, NULL);
+	    }
 	    case POLYGON: {
 	       pointlist tmppts;
 	       int tmpnum;
@@ -1063,19 +1178,36 @@ short redo_one_action()
 	       calcspline(espline);
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
-	    case ARC: {
-	       arcinfo tmpinfo;
-	       arcptr earc = (arcptr)(erec->element);
-	       tmpinfo.angle1 = earc->angle1; 
-	       tmpinfo.angle2 = earc->angle2; 
-	       tmpinfo.radius = earc->radius; 
-	       tmpinfo.yaxis = earc->yaxis; 
-	       earc->angle1 = erec->save.arcspecs->angle1;
-	       earc->angle2 = erec->save.arcspecs->angle2;
-	       earc->radius = erec->save.arcspecs->radius;
-	       earc->yaxis = erec->save.arcspecs->yaxis;
-	       *(erec->save.arcspecs) = tmpinfo;
-	       calcarc(earc);
+	    case PATH: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly;
+	       splineptr espline;
+	       pathinfo *tmpinfo = erec->save.pathspecs;
+	       pathptr epath = (pathptr)(erec->element);
+	       for (i = 0; i < epath->parts; i++) {
+		  genericptr ggen = *(epath->plist + i);
+		  switch (ELEMENTTYPE(ggen)) {
+		     case POLYGON:
+			epoly = (polyptr)ggen;
+		        tmppts = epoly->points;
+			tmpnum = epoly->number;
+			epoly->points = (erec->save.pathspecs + i)->points;
+			epoly->number = (erec->save.pathspecs + i)->number;
+			(erec->save.pathspecs + i)->points = tmppts;
+			(erec->save.pathspecs + i)->number = tmpnum;
+			break;
+		     case SPLINE:
+			espline = (splineptr)ggen;
+			tmppts = copypoints((pointlist)espline->ctrl, 4);
+			for (j = 0; j < 4; j++)
+			   espline->ctrl[j] = *((erec->save.pathspecs + i)->points + j);
+			free((erec->save.pathspecs + i)->points);
+		        (erec->save.pathspecs + i)->points = tmppts;
+			calcspline(espline);
+			break;
+		  }
+	       }
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -1167,7 +1299,9 @@ short redo_one_action()
 
       case XCF_Move:
 	 delta = (XPoint *)thisrecord->undodata;
+	 select_connected_pins();
 	 placeselects(delta->x, delta->y, NULL);
+	 reset_connected_pins();
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1176,7 +1310,17 @@ short redo_one_action()
 	 break;
    }
 
-   eventmode = NORMAL_MODE;	/* Set on a per-event-type basis? */
+   /* Does this need to be set on a per-event-type basis? */
+   switch (savemode) {
+      case CATALOG_MODE:
+      case CATTEXT_MODE:
+	 eventmode = CATALOG_MODE;
+	 break;
+      default:
+	 eventmode = NORMAL_MODE;
+	 break;
+   }
+
    areawin = savewindow;
 
    return thisrecord->idx;
@@ -1283,7 +1427,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Edit:
 	 erec = (editelement *)thisrecord->undodata;
-	 free_editelement(erec);
+	 free_editelement(thisrecord);
 	 break;
 
       case XCF_Copy:
@@ -1325,7 +1469,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
 void free_undo_record(Undoptr thisrecord)
 {
-   Undoptr nextrecord, lastrecord;
+  /* Undoptr nextrecord, lastrecord; (jdk) */
 
    /* Reset the master list pointers */
 
@@ -1353,7 +1497,7 @@ void free_undo_record(Undoptr thisrecord)
 
 void free_redo_record(Undoptr thisrecord)
 {
-   Undoptr nextrecord, lastrecord;
+  /* Undoptr nextrecord, lastrecord; (jdk) */
 
    /* Reset the master list pointers */
 
