@@ -36,6 +36,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <locale.h>
+#include <ctype.h>
 #ifndef XC_WIN32
 #include <unistd.h>   /* for unlink() */
 
@@ -91,13 +92,6 @@
 /*----------------------------------------------------------------------*/
 #include "prototypes.h"
 
-#ifdef HAVE_XPM
-#ifndef XC_WIN32
-#include <X11/xpm.h>
-#endif
-#include "lib/pixmaps/xcircuit.xpm"
-#endif
-
 /*----------------------------------------------------------------------*/
 /* Global Variable definitions						*/
 /*----------------------------------------------------------------------*/
@@ -176,11 +170,11 @@ void addtocolorlist(xcWidget button, int cvalue)
 
 int addnewcolorentry(int ccolor)
 {
-   xcWidget colormenu, newbutton;
+  xcWidget newbutton; /* colormenu, (jdk) */
 #ifndef XC_WIN32
-   Arg wargs[2];
+  /* Arg wargs[2]; (jdk) */
 #endif
-   int i, n = 0;
+  int i; /* , n = 0; (jdk) */
 
    /* check to see if entry is already in the color list */
 
@@ -306,10 +300,11 @@ u_short getchanges(objectptr thisobj)
 
 u_short countchanges(char **promptstr)
 {
-   int slen = 1, i, j;
+  int slen = 1, i; /* , j; (jdk) */
    u_short locchanges, changes = 0, words = 1;
    objectptr thisobj;
    char *fname;
+   TechPtr ns;
 
    if (promptstr != NULL) slen += strlen(*promptstr);
 
@@ -333,22 +328,20 @@ u_short countchanges(char **promptstr)
 
    /* Check all library objects for unsaved changes */
 
-
-   for (i = 0; i < xobjs.numlibs; i++) {
-      lib_set_changes(i);
-      if ((xobjs.userlibs[i].flags & LIBRARY_CHANGED) != 0) {
+   for (ns = xobjs.technologies; ns != NULL; ns = ns->next) {
+      tech_set_changes(ns);
+      if ((ns->flags & LIBRARY_CHANGED) != 0) {
 	 changes++;
 	 if (promptstr != NULL) {
-	    if (xobjs.userlibs[i].filename != NULL)
-	          fname = xobjs.userlibs[i].filename;
-	    else
-	         fname = xobjs.libtop[i + LIBRARY]->thisobject->name;
-	    slen += strlen(fname) + 2;
-	    *promptstr = (char *)realloc(*promptstr, slen);
-	    if ((words % 8) == 0) strcat(*promptstr, ",\n");
-	    else if (changes > 0) strcat(*promptstr, ", ");
-	    strcat(*promptstr, fname);
-	    words++;
+	    fname = ns->filename;
+	    if (fname != NULL) {
+	       slen += strlen(fname) + 2;
+	       *promptstr = (char *)realloc(*promptstr, slen);
+	       if ((words % 8) == 0) strcat(*promptstr, ",\n");
+	       else if (changes > 0) strcat(*promptstr, ", ");
+	       strcat(*promptstr, fname);
+	       words++;
+	    }
 	 }
       }
    }
@@ -388,6 +381,7 @@ void quitcheck(xcWidget w, caddr_t clientdata, caddr_t calldata)
       Tcl_Eval(xcinterp, promptstr);
       Tcl_Eval(xcinterp, ".query.bbar.okay configure -command {quitnocheck}");
       Tcl_Eval(xcinterp, "wm deiconify .query");
+      Tcl_Eval(xcinterp, "raise .query");
       free(promptstr);
    }
    else {
@@ -499,7 +493,8 @@ void outputpopup(xcWidget button, caddr_t clientdata, caddr_t calldata)
 /* For formatted strings, format first into _STR   */
 /*-------------------------------------------------*/
 
-xcTimeOutProc clrmessage(caddr_t clientdata)
+/* XXX it looks like this routine is an orphan */
+void clrmessage(caddr_t clientdata)
 {
    char buf1[50], buf2[50];
 
@@ -700,7 +695,7 @@ int findnearcolor(XColor *cvexact)
 int xc_getlayoutcolor(int cidx)
 {
    XColor loccolor;
-   int i, locidx;
+   int locidx; /* i, (jdk) */
 
    loccolor.pixel = cidx;
    loccolor.flags = DoRed | DoGreen | DoBlue;
@@ -829,7 +824,7 @@ int rgb_alloccolor(int red, int green, int blue)
 int rgb_alloccolor(int red, int green, int blue)
 {
    XColor newcolor;
-   int i, pixval, tableidx;
+   int pixval, tableidx; /* i, (jdk) */
 
    /* if color is not already in list, try to allocate it; if allocation */
    /* fails, grab the closest match in the colormap.			 */
@@ -1030,8 +1025,10 @@ XCWindowData *create_new_window()
 
    newwindow = (XCWindowData *)malloc(sizeof(XCWindowData));
 
+#ifndef TCL_WRAPPER
 #ifdef HAVE_XPM
    newwindow->toolbar_on = True;
+#endif
 #endif
 
    newwindow->area = (xcWidget)NULL;
@@ -1063,11 +1060,14 @@ XCWindowData *create_new_window()
    newwindow->hierstack = NULL;
    initmem(newwindow->editstack);
    newwindow->pinpointon = False;
+   newwindow->pinattach = False;
    newwindow->buschar = '(';	/* Vector notation for buses */
    newwindow->defaultcursor = &CROSS;
    newwindow->event_mode = NORMAL_MODE;
-   newwindow->attachto = FALSE;
+   newwindow->attachto = -1;
    newwindow->color = DEFAULTCOLOR;
+   newwindow->gccolor = 0;
+   newwindow->gctype = 0;
    newwindow->time_id = 0;
    newwindow->vscale = 1;
    newwindow->pcorner.x = newwindow->pcorner.y = 0;
@@ -1126,11 +1126,13 @@ void pre_initialize()
    xobjs.pagelist[0]->gridspace = DEFAULTGRIDSPACE;
    xobjs.pagelist[0]->snapspace = DEFAULTSNAPSPACE;
    xobjs.pagelist[0]->drawingscale.x = xobjs.pagelist[0]->drawingscale.y = 1;
-   xobjs.pagelist[0]->coordstyle = FRAC_INCH;
+   xobjs.pagelist[0]->coordstyle = INTERNAL;
    xobjs.pagelist[0]->pagesize.x = 612;
    xobjs.pagelist[0]->pagesize.y = 792;
 
    xobjs.hold = TRUE;
+   xobjs.showtech = FALSE;
+   xobjs.suspend = (char)0;	/* Suspend graphics until finished with startup */
    xobjs.new_changes = 0;
    xobjs.do_replace = -1;	/* default: replace NONE */
    xobjs.filefilter = TRUE;
@@ -1139,6 +1141,7 @@ void pre_initialize()
    signal(SIGINT, dointr);
    printtime_id = 0;
 
+   xobjs.technologies = NULL;
    xobjs.undostack = NULL;
    xobjs.redostack = NULL;
 
@@ -1155,11 +1158,9 @@ void pre_initialize()
    xobjs.fontlib.number = 0;
    xobjs.userlibs = (Library *) malloc(xobjs.numlibs * sizeof(Library));
    for (i = 0; i < xobjs.numlibs; i++) {
-      xobjs.userlibs[i].filename = NULL;
       xobjs.userlibs[i].library = (objectptr *) malloc(sizeof(objectptr));
       xobjs.userlibs[i].instlist = NULL;
       xobjs.userlibs[i].number = 0;
-      xobjs.userlibs[i].flags = (char)0;
    }
    xobjs.imagelist = NULL;
    xobjs.images = 0;
@@ -1201,10 +1202,10 @@ static Tcl_ObjType tclHandleType = {
 
 static void
 UpdateStringOfHandle(objPtr)
-    register Tcl_Obj *objPtr;   /* Int object whose string rep to update. */
+    Tcl_Obj *objPtr;   /* Int object whose string rep to update. */
 {
     char buffer[TCL_INTEGER_SPACE];
-    register int len;
+    int len;
 
     sprintf(buffer, "H%08lX", objPtr->internalRep.longValue);
     len = strlen(buffer);
@@ -1219,12 +1220,12 @@ UpdateStringOfHandle(objPtr)
 static int
 SetHandleFromAny(interp, objPtr)
     Tcl_Interp *interp;         /* Used for error reporting if not NULL. */
-    register Tcl_Obj *objPtr;   /* The object to convert. */
+    Tcl_Obj *objPtr;   /* The object to convert. */
 {  
     Tcl_ObjType *oldTypePtr = objPtr->typePtr;
     char *string, *end;
     int length;
-    register char *p;
+    char *p;
     long newLong;
     pushlistptr newstack = NULL;
 
@@ -1239,7 +1240,6 @@ nexthier:
             Tcl_ResetResult(interp);
             Tcl_AppendToObj(Tcl_GetObjResult(interp), 
 		"handle is identified by leading H and hexidecimal value only", -1);
-            TclCheckBadOctal(interp, string);
         }
         free_stack(&newstack);
         return TCL_ERROR;
@@ -1258,7 +1258,6 @@ nexthier:
             sprintf(buf, "expected handle but got \"%.50s\"", string);
             Tcl_ResetResult(interp);
             Tcl_AppendToObj(Tcl_GetObjResult(interp), buf, -1);
-            TclCheckBadOctal(interp, string);
         }
         free_stack(&newstack);
         return TCL_ERROR;
@@ -1354,9 +1353,9 @@ nexthier:
 
 Tcl_Obj *
 Tcl_NewHandleObj(optr)
-    register void *optr;      /* Int used to initialize the new object. */
+    void *optr;      /* Int used to initialize the new object. */
 {
-    register Tcl_Obj *objPtr;
+    Tcl_Obj *objPtr;
 
     objPtr = Tcl_NewObj();
     objPtr->bytes = NULL;
@@ -1371,10 +1370,10 @@ Tcl_NewHandleObj(optr)
 int
 Tcl_GetHandleFromObj(interp, objPtr, handlePtr)
     Tcl_Interp *interp;		/* Used for error reporting if not NULL. */
-    register Tcl_Obj *objPtr;	/* The object from which to get a int. */
-    register void **handlePtr;	/* Place to store resulting int. */
+    Tcl_Obj *objPtr;	/* The object from which to get a int. */
+    void **handlePtr;	/* Place to store resulting int. */
 {
-    register long l;
+    long l;
     int result;
 
     if (objPtr->typePtr != &tclHandleType) {
@@ -1468,7 +1467,7 @@ void post_initialize()
 
    xobjs.save_interval = appdata.timeout;
    xobjs.timeout_id = xcAddTimeOut(app, 60000 * xobjs.save_interval,
-	(xcTimeOutProc)savetemp, NULL);
+	savetemp, NULL);
 }
 
 /*----------------------------------------------------------------------*/
