@@ -929,7 +929,7 @@ void highlightexcl(xcWidget widget, int func, int value)
 
 	 default:
 	    for (titem = ToolBar; titem < ToolBar + toolbuttons; titem++) {
-	       if (func == (int)(titem->passeddata)) {
+	       if (func == (pointertype)(titem->passeddata)) {
                   self = XtNameToWidget(parent, titem->name);
 	          break;
 	       }
@@ -1057,8 +1057,12 @@ int createlibrary(Boolean force)
    xobjs.userlibs[libnum - LIBRARY].library = (objectptr *) malloc(sizeof(objectptr));
    xobjs.userlibs[libnum - LIBRARY].number = 0;
    xobjs.userlibs[libnum - LIBRARY].instlist = NULL;
+
+   /* To-do:  initialize technology list */
+   /*
    xobjs.userlibs[libnum - LIBRARY].filename = NULL;
    xobjs.userlibs[libnum - LIBRARY].flags = (char)0;
+   */
 
    /* Previously last button becomes new library pointer */
 
@@ -1146,36 +1150,6 @@ void renamepage(short pagenumber)
    }
    else if (button == NULL)
       Fprintf(stderr, "Error:  No Button Widget named \"%9s\"\n", bname);
-
-   /* The GUI has been taken care of, but the page itself might not exist */
-   if (!thisinst) return;
-
-   /* If the page had a master but now has a different	*/
-   /* name, disconnect the schematics.			*/
-
-   if (thisinst->thisobject->schemtype == SECONDARY) {
-      if (strcmp(thisinst->thisobject->name, thisinst->thisobject->symschem->name)) {
-	 thisinst->thisobject->symschem = NULL;
-	 thisinst->thisobject->schemtype = PRIMARY;
-      }
-      Wprintf("Page disconnected from master schematic.");
-   }
-
-   /* Conversely, if the page has the same name as another page, the	*/
-   /* schematics are considered connected.				*/
-
-   for (page = 0; page < xobjs.pages; page++) {
-      objinstptr pageinst;
-      if (page == pagenumber) continue;
-      pageinst = xobjs.pagelist[page]->pageinst;
-      if (pageinst && !strcmp(pageinst->thisobject->name,
-		thisinst->thisobject->name)) {
-	 thisinst->thisobject->symschem = pageinst->thisobject;
-	 thisinst->thisobject->schemtype = SECONDARY;
-         Wprintf("Page connected to master schematic.");
-	 break;
-      }
-   }
 }
 
 /*--------------------------------------------------------------*/
